@@ -2,7 +2,7 @@ package Net::GitHub;
 
 use Net::GitHub::V3;
 
-our $VERSION = '0.67_01';
+our $VERSION = '0.92_01';
 our $AUTHORITY = 'cpan:FAYLAND';
 
 sub new {
@@ -26,9 +26,21 @@ Net::GitHub - Perl Interface for github.com
         login => 'fayland', pass => 'secret'
     );
 
-    # Pass api_url for GitHub Enterprise installations
+    # If you use two factor authentication you can pass in the OTP. Do
+    # note that OTPs expire quickly and you will need to generate an oauth
+    # token to do anything non-trivial.
+    my $github = Net::GitHub->new(
+        login =>   'fayland',
+        pass =>    'secret',
+        otp =>     '123456',
+    );
+
+    # Pass api_url for GitHub Enterprise installations. Do not include a
+    # trailing slash
     my $github = Net::GitHub->new(  # Net::GitHub::V3
-        login => 'fayland', pass => 'secret',  api_url => 'https://gits.aresweet.com/api/v3'
+        login =>   'fayland',
+        pass =>    'secret',
+        api_url => 'https://gits.aresweet.com/api/v3'
     );
 
     # suggested
@@ -59,6 +71,8 @@ Check L<http://developer.github.com/> for more details of the GitHub APIs.
 
 Read L<Net::GitHub::V3> for API usage.
 
+Read L<Net::GitHub::V4> for GitHub GraphQL API.
+
 If you prefer object oriented way, L<Pithub> is 'There is more than one way to do it'.
 
 =head2 FAQ
