@@ -1,16 +1,19 @@
 package Net::GitHub::V3::Query;
 
-our $VERSION = '0.67';
+our $VERSION = '0.92_01';
 our $AUTHORITY = 'cpan:FAYLAND';
 
 use URI;
-use JSON::Any;
+use JSON::MaybeXS;
 use MIME::Base64;
 use LWP::UserAgent;
 use HTTP::Request;
 use Carp qw/croak/;
 use URI::Escape;
-use Types::Standard qw(Str Bool InstanceOf);
+use Types::Standard qw(Int Str Bool InstanceOf Object);
+use Cache::LRU;
+
+use Scalar::Util qw(looks_like_number);
 
 use Moo::Role;
 
@@ -19,6 +22,7 @@ use Moo::Role;
 # Authentication
 has 'login'  => ( is => 'rw', isa => Str, predicate => 'has_login' );
 has 'pass'  => ( is => 'rw', isa => Str, predicate => 'has_pass' );
+has 'otp'  => ( is => 'rw', isa => Str, predicate => 'has_otp' );
 has 'access_token' => ( is => 'rw', isa => Str, predicate => 'has_access_token' );
 
 # return raw unparsed JSON
@@ -35,14 +39,24 @@ has 'next_url'  => ( is => 'rw', isa => Str, predicate => 'has_next_page',  clea
 has 'last_url'  => ( is => 'rw', isa => Str, predicate => 'has_last_page',  clearer => 'clear_last_url' );
 has 'first_url' => ( is => 'rw', isa => Str, predicate => 'has_first_page', clearer => 'clear_first_url' );
 has 'prev_url'  => ( is => 'rw', isa => Str, predicate => 'has_prev_page',  clearer => 'clear_prev_url' );
+has 'per_page'  => ( is => 'rw', isa => Str, default => 100 );
+has 'total_pages'  => ( is => 'rw', isa => Str, default => 0 );
 
 # Error handle
 has 'RaiseError' => ( is => 'rw', isa => Bool, default => 1 );
 
+# Rate limits
+has 'rate_limit' => ( is => 'rw', isa => Int, default => 0 );
+has 'rate_limit_remaining' => ( is => 'rw', isa => Int, default => 0 );
+has 'rate_limit_reset' => ( is => 'rw', isa => Str, default => 0 );
+
 # optional
 has 'u'  => (is => 'rw', isa => Str);
 has 'repo' => (is => 'rw', isa => Str);
 
+# accept version
+has 'accept_version' => (is => 'rw', isa => Str, default => '');
+
 has 'is_main_module' => (is => 'ro', isa => Bool, default => 0);
 sub set_default_user_repo {
     my ($self, $user, $repo) = @_;
@@ -72,7 +86,7 @@ sub set_default_user_repo {
 sub args_to_pass {
     my $self = shift;
     my $ret;
-    foreach my $col ('login', 'pass', 'access_token', 'raw_string', 'raw_response', 'api_url', 'api_throttle', 'u', 'repo', 'next_url', 'last_url', 'first_url', 'prev_url') {
+    foreach my $col ('login', 'pass', 'otp', 'access_token', 'raw_string', 'raw_response', 'api_url', 'api_throttle', 'u', 'repo', 'next_url', 'last_url', 'first_url', 'prev_url', 'per_page', 'ua') {
         my $v = $self->$col;
         $ret->{$col} = $v if defined $v;
     }
@@ -85,7 +99,7 @@ has 'ua' => (
     lazy    => 1,
     default => sub {
         LWP::UserAgent->new(
-            agent       => "perl-net-github $VERSION",
+            agent       => "perl-net-github/$VERSION",
             cookie_jar  => {},
             keep_alive  => 4,
             timeout     => 60,
@@ -95,13 +109,24 @@ has 'ua' => (
 
 has 'json' => (
     is => 'ro',
-    isa => InstanceOf['JSON::Any'],
+    isa => Object, # InstanceOf['JSON::MaybeXS'],
     lazy => 1,
     default => sub {
-        return JSON::Any->new( utf8 => 1 );
+        return JSON::MaybeXS->new( utf8 => 1 );
     }
 );
 
+has 'cache' => (
+  isa => InstanceOf['Cache::LRU'],
+  is => 'rw',
+  lazy => 1,
+  default => sub {
+    Cache::LRU->new(
+      size => 200
+    );
+  }
+);
+
 sub query {
     my $self = shift;
 
@@ -124,28 +149,51 @@ sub query {
     } elsif ($self->has_login and $self->has_pass) {
         my $auth_basic = $self->login . ':' . $self->pass;
         $ua->default_header('Authorization', 'Basic ' . encode_base64($auth_basic));
+        if ($self->has_otp) {
+            $ua->default_header('X-GitHub-OTP', $self->otp);
+        }
     }
 
     $url = $self->api_url . $url unless $url =~ /^https\:/;
+    if ($request_method eq 'GET') {
+        if ($url !~ /per_page=\d/) {
+            ## auto add per_page in url for GET no matter it supports or not
+            my $uri = URI->new($url);
+            my %query_form = $uri->query_form;
+            $query_form{per_page} ||= $self->per_page;
+            $uri->query_form(%query_form);
+            $url = $uri->as_string;
+        }
+    }
 
     print STDERR ">>> $request_method $url\n" if $ENV{NG_DEBUG};
     my $req = HTTP::Request->new( $request_method, $url );
     $req->accept_decodable;
     if ($data) {
-        my $json = $self->json->objToJson($data);
+        my $json = $self->json->encode($data);
         print STDERR ">>> $data\n" if $ENV{NG_DEBUG} and $ENV{NG_DEBUG} > 1;
         $req->content($json);
     }
     $req->header( 'Content-Length' => length $req->content );
 
-    my $res = $ua->request($req);
+    # if preview API, specify a custom media type to Accept header
+    # https://developer.github.com/v3/media/
+    $req->header( 'Accept' => sprintf("application/vnd.github.%s.param+json", $self->accept_version) )
+        if $self->accept_version;
+
+    my $res = $self->_make_request($req);
+
+    # get the rate limit information from the http response headers
+    $self->rate_limit( $res->header('x-ratelimit-limit') );
+    $self->rate_limit_remaining( $res->header('x-ratelimit-remaining') );
+    $self->rate_limit_reset( $res->header('x-ratelimit-reset') );
 
     # Slow down if we're approaching the rate limit
     # By the way GitHub mistakes days for minutes in their documentation --
     # the rate limit is per minute, not per day.
     if ( $self->api_throttle ) {
-        sleep 2 if (($res->header('x-ratelimit-remaining') || 0)
-            < ($res->header('x-ratelimit-limit') || 60) / 2);
+        sleep 2 if (($self->rate_limit_remaining || 0)
+            < ($self->rate_limit || 60) / 2);
     }
 
     print STDERR "<<< " . $res->decoded_content . "\n" if $ENV{NG_DEBUG} and $ENV{NG_DEBUG} > 1;
@@ -154,7 +202,7 @@ sub query {
 
     if ($res->header('Content-Type') and $res->header('Content-Type') =~ 'application/json') {
         my $json = $res->decoded_content;
-        $data = eval { $self->json->jsonToObj($json) };
+        $data = eval { $self->json->decode($json) };
         unless ($data) {
             # We tolerate bad JSON for errors,
             # otherwise we just rethrow the JSON parsing problem.
@@ -166,7 +214,19 @@ sub query {
     }
 
     if ( $self->RaiseError ) {
-        croak $data->{message} if not $res->is_success and ref $data eq 'HASH' and exists $data->{message}; # for 'Client Errors'
+        # check for 'Client Errors'
+        if (not $res->is_success and ref $data eq 'HASH' and exists $data->{message}) {
+            my $message = $data->{message};
+
+            # Include any additional error information that was returned by the API
+            if (exists $data->{errors}) {
+                $message .= ': '.join(' - ',
+                                     map { $_->{message} }
+                                     grep { exists $_->{message} }
+                                     @{ $data->{errors} });
+            }
+            croak $message;
+        }
     }
 
     $self->_clear_pagination;
@@ -184,11 +244,45 @@ sub query {
     return $data;
 }
 
+sub set_next_page {
+    my ($self, $page) = @_;
+
+    if( ! looks_like_number($page) ){
+	    croak "Trying to set_next_page to $page, and not a number\n";
+    }
+
+    if( $page > $self->total_page && $page > 0 ){
+	    return 0;
+    }
+
+    my $temp_url = $self->next_url;
+    $temp_url =~ s/([&?])page=[0-9]+([&?]*)/$1page=$page$2/;
+
+    $self->next_url( $temp_url );
+
+    return 1;
+}
+
 sub next_page {
     my $self = shift;
     return $self->query($self->next_url);
 }
 
+sub prev_page {
+    my $self = shift;
+    return $self->query($self->prev_url);
+}
+
+sub first_page {
+    my $self = shift;
+    return $self->query($self->first_url);
+}
+
+sub last_page {
+    my $self = shift;
+    return $self->query($self->last_url);
+}
+
 sub _clear_pagination {
     my $self = shift;
     foreach my $page (qw/first last prev next/) {
@@ -212,11 +306,50 @@ sub _extract_link_url {
 
         my $url_attr = $rel . "_url";
         $self->$url_attr($link_url);
+
+        # Grab, and expose, some additional header information
+	if( $rel eq "last" ){
+	    $link_url =~ /[\&?]page=([0-9]*)[\&?]*/;
+	    $self->total_pages( $1 );
+	}
     }
 
     return 1;
 }
 
+sub _make_request {
+  my($self, $req) = @_;
+
+  my $cached_res = $self->_get_shared_cache($req->uri);
+
+  if ($cached_res) {
+    $req->header("If-None-Match" => $cached_res->header("ETag"));
+    my $res = $self->ua->request($req);
+
+    if ($res->code == 304) {
+      return $cached_res;
+    }
+
+    $self->_set_shared_cache($req->uri, $res);
+
+    return $res;
+  } else {
+    my $res = $self->ua->request($req);
+    $self->_set_shared_cache( $req->uri, $res);
+    return $res;
+  }
+}
+
+sub _get_shared_cache {
+  my ($self, $uri) = @_;
+  return $self->cache->get($uri);
+}
+
+sub _set_shared_cache {
+  my($self, $uri, $response) = @_;
+  $self->cache->set($uri, $response);
+}
+
 ## build methods on fly
 sub __build_methods {
     my $package = shift;
@@ -229,6 +362,7 @@ sub __build_methods {
         my $args = $v->{args} || 0; # args for ->query
         my $check_status = $v->{check_status};
         my $is_u_repo = $v->{is_u_repo}; # need auto shift u/repo
+        my $preview_version = $v->{preview};
 
         no strict 'refs';
         no warnings 'once';
@@ -247,6 +381,9 @@ sub __build_methods {
             my @uargs = splice(@_, 0, $n);
             my $u = sprintf($url, @uargs);
 
+            # if preview API, set preview version
+            $self->accept_version($preview_version) if $preview_version;
+
             # args for json data POST
             my @qargs = $args ? splice(@_, 0, $args) : ();
             if ($check_status) { # need check Response Status
@@ -292,7 +429,7 @@ set Authentication and call API
 
 =item access_token
 
-either set access_token from OAuth or login:pass for Basic Authentication
+Either set access_token from OAuth or login:pass for Basic Authentication
 
 L<http://developer.github.com/>
 
@@ -302,6 +439,25 @@ L<http://developer.github.com/>
 
 =item api_throttle
 
+API throttling is enabled by default, set api_throttle to 0 to disable it.
+
+=item rate_limit
+
+The maximum number of queries allowed per hour. 60 for anonymous users and
+5,000 for authenticated users.
+
+=item rate_limit_remaining
+
+The number of requests remaining in the current rate limit window.
+
+=item rate_limit_reset
+
+The time the current rate limit resets in UTC epoch seconds.
+
+=item last_page
+
+Denotes the index of the last page in the pagination
+
 =item RaiseError
 
 =back
@@ -318,6 +474,23 @@ Refer L<Net::GitHub::V3>
 
 Calls C<query> with C<next_url>. See L<Net::GitHub::V3>
 
+=item prev_page
+
+Calls C<query> with C<prev_url>. See L<Net::GitHub::V3>
+
+=item first_page
+
+Calls C<query> with C<first_url>. See L<Net::GitHub::V3>
+
+=item last_page
+
+Calls C<query> with C<last_url>. See L<Net::GitHub::V3>
+
+=item set_next_page
+
+Adjusts next_url to be a new url in the pagination space
+I.E. you are jumping to a new index in the pagination
+
 =back
 
 =head3 NG_DEBUG
